library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;
use ieee.std_logic_arith.all;

entity processore is
port ( rst,clk : in std_logic;
	addr: out integer range 0 to 65535;
	dout: out  std_logic_vector(31 downto 0);
	din: in  std_logic_vector(31 downto 0);
	memread,memwrite: out std_logic);

end processore;

architecture beh of processore is
type stato is (fetch,decode,ex_r,wb_r,
		ex_b,
		ex_l,mm_l,wb_l,
		ex_s,mm_s, ex_i, wb_i);
signal s: stato;
signal i: std_logic_vector(5 downto 0);
  type rf is array( 0 to 31) of std_logic_vector(31 downto 0);
  signal regfile: rf;
  signal pc: integer range 0 to 65535;
  signal a,b,r: std_logic_vector(31 downto 0);

begin

process(rst,clk)
  variable  ts: stato;
 begin
 if rst = '1' then ts := fetch;
 elsif clk'event and clk='0' then
   case s is
    when fetch => ts := decode;
    when decode => case i is
    		    when "000000" => ts:= ex_r;
    		    when "000001"|"000010"|"000011" => ts:= ex_i;
    		    when "010000" => ts := ex_b;
    		    when "100000" => ts := ex_l;
    		    when "101000" => ts := ex_s;
    		    when others => ts := fetch;
    		    end case;
    when ex_r => ts := wb_r;
    when ex_i => ts := wb_i; 
    when ex_l => ts := mm_l;
    when mm_l => ts := wb_l;
    when ex_s => ts := mm_s;
    when others => ts := fetch;
   end case;
  end if;
  
  s <= ts after 1 ns;
  
  end process;
  
 
 process(s,r,b)
   begin
  
  case s is 
    
   when fetch =>
   	addr <= pc;
   	memread <= '1';
   	memwrite <= '0';
   	dout <= (dout'range => 'Z');
   	
   when mm_l =>
   	addr <= conv_integer(r);
   	memread <= '1';
   	memwrite <= '0';
   	dout <= (dout'range => 'Z');
  	 
	 
    when mm_s => 
        addr <= conv_integer(r);
         dout <= b;
         memread <= '0';
         memwrite <= '1';
        
   when others =>
           memread <= '0';
           memwrite <= '0';
           dout <= (dout'range => 'Z');  
  end case;                     	  	        
 
 end process; 
 
 
 process(rst,clk)
  
  variable istr,d,imm_est: std_logic_vector(31 downto 0);
  begin
  
  if rst = '1' then    
     pc<=0;
    for i in 0 to 31 loop
     regfile(i) <= (regfile(i)'range => '0');
    end loop;
   
  elsif clk'event and clk='0' then
      
  case s is 
   
  
   when fetch =>
   	istr:= din;
   	pc<= pc+1 after 1 ns;
   	i <= istr(31 downto 26) after 1 ns;
   when decode =>
     a<= regfile(conv_integer(istr(25 downto 21)));
     b<= regfile(conv_integer(istr(20 downto 16)));
     -- calcolo dell'immediato esteso in segno
     if istr(15)= '1' then 
          imm_est(31 downto 16) := (imm_est(31 downto 16)'range =>'1');
     
     else imm_est(31 downto 16) := (imm_est(31 downto 16)'range =>'0');
     end if;
     imm_est(15 downto 0) := istr(15 downto 0);                 
        
   when ex_r =>
        case istr(5 downto 0) is
          when "000000" => r <= a + b;
          when "000001" => r <= a - b;
          when "000010" => r <= a and b;
          when "000100" => r <= a or b;
          when "001000" => r <= a  xor b;
          when others => r <= not a;
         end case;
   when ex_i => 
        case istr(31 downto 26) is
          when "000001" => r <= a + imm_est;
          when "000010" => r <= a - imm_est;
          when "000011" => r<= a and imm_est;
          when others =>  r <= not a;
        end case;      
   when wb_r => 
        regfile(conv_integer(istr(15 downto 11))) <= r;
   when wb_i => regfile(conv_integer(istr(20 downto 16))) <= r;
        
   when ex_b =>
         if a=b then pc <= pc+conv_integer(imm_est);
         end if;
        
   when ex_l =>
     	r <= a + imm_est;
   when mm_l =>
   	d:=  din;
   when wb_l =>  
   	
    regfile(conv_integer(istr(20 downto 16))) <= d;
   	
   when ex_s =>
  	 r <= a + imm_est;
   when mm_s => 
        
        
         
   when others => null;
           
  end case;                     	  	        
 end if;
  
  
  
  
 end process; 
 
 
 
 end beh;   









